/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package explorviz.hpc_monitoring.connector;

import java.io.*;
import kieker.analysis.IProjectContext;
import kieker.analysis.plugin.annotation.*;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.common.configuration.Configuration;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.common.record.IMonitoringRecord;
import explorviz.hpc_monitoring.byteaccess.UnsafeBits;

/**
 * A plugin used for kieker in the cloud.
 * All incoming events are put into a RabbitMQ, but are also passed to an output
 * port that can be used for
 * testing purposes.
 * 
 * @author Santje Finke
 * 
 * @since 1.8
 * 
 */
@Plugin(description = "A filter that writes all incoming events into a specified queue on a specified RabbitMQServer", outputPorts = { @OutputPort(name = TCPConnector.OUTPUT_PORT_NAME, eventTypes = { Object.class }, description = "Provides each incoming object") }, configuration = {
        @Property(name = TCPConnector.CONFIG_PROPERTY_NAME_PROVIDER, defaultValue = "localhost"),
        @Property(name = TCPConnector.CONFIG_PROPERTY_NAME_QUEUE, defaultValue = "master"),
        @Property(name = TCPConnector.CONFIG_PROPERTY_PASSWORD, defaultValue = "guest"),
        @Property(name = TCPConnector.CONFIG_PROPERTY_USER, defaultValue = "guest") })
public class TCPConnector extends AbstractFilterPlugin {
    private static final int   MESSAGE_BUFFER_SIZE            = 1500;

    /**
     * The name of the input port receiving the incoming events.
     */
    public static final String INPUT_PORT_NAME_INVALID_TRACES = "inputInvalidTraces";
    public static final String INPUT_PORT_NAME_VALID_TRACES   = "inputValidTraces";

    /**
     * The name of the output port passing the incoming events.
     */
    public static final String OUTPUT_PORT_NAME               = "relayedEvents";
    /**
     * The name of the property determining the address of the used Server.
     */
    public static final String CONFIG_PROPERTY_NAME_PROVIDER  = "providerUrl";
    /**
     * The name of the property determining the name of the Queue.
     */
    public static final String CONFIG_PROPERTY_NAME_QUEUE     = "queueName";

    /**
     * The username that is used to connect to a queue.
     */
    public static final String CONFIG_PROPERTY_USER           = "guest";
    /**
     * The password that is used to connect to a queue.
     */
    public static final String CONFIG_PROPERTY_PASSWORD       = "guest";

    private static final Log   LOG                            = LogFactory
                                                                      .getLog(TCPConnector.class);

    private final String       providerUrl;

    private final byte[]       validTracesMessages            = new byte[MESSAGE_BUFFER_SIZE];
    private int                validTracesMessagesOffset      = 0;

    private final byte[]       invalidTracesMessages          = new byte[MESSAGE_BUFFER_SIZE];
    private int                invalidTracesMessagesOffset    = 0;

    public TCPConnector(final Configuration configuration,
            final IProjectContext projectContext) {
        super(configuration, projectContext);
        providerUrl = configuration
                .getStringProperty(CONFIG_PROPERTY_NAME_PROVIDER);
        try {
            connect();
        }
        catch (final IOException e) {
            e.printStackTrace();
        }
    }

    private void connect() throws IOException {}

    /**
     * This method represents the input port of this filter.
     * 
     * @param event
     *            The next event.
     */
    @InputPort(name = INPUT_PORT_NAME_VALID_TRACES, eventTypes = { Object.class }, description = "Receives incoming objects to be forwarded to a queue")
    public final void inputValidTraces(final Object monitoringRecord) {
        try {
            // if (monitoringRecord instanceof IMonitoringRecord) {
            // byte[] message2 = toByteArray((IMonitoringRecord)
            // monitoringRecord);

            final ByteArrayOutputStream boas = new ByteArrayOutputStream();

            final ObjectOutputStream out = new ObjectOutputStream(boas);
            out.writeObject(monitoringRecord);
            out.close();

            final byte[] message2 = boas.toByteArray(); // TODO optimize

            System.arraycopy(message2, 0, validTracesMessages,
                    validTracesMessagesOffset, message2.length);
            validTracesMessagesOffset += message2.length;

            if ((validTracesMessagesOffset > (MESSAGE_BUFFER_SIZE - MESSAGE_BUFFER_SIZE))) { // TODO
                // unsafe
                // 200
                // ||
                // writeQueue.isEmpty()
                UnsafeBits.putInt(validTracesMessages,
                        validTracesMessagesOffset, -1);
                sendMessage(validTracesMessages, "validTracesMaster");
                validTracesMessagesOffset = 0;
            }
        }
        catch (final IOException e) {
            LOG.error("Error sending record", e);
        }
    }

    /**
     * This method represents the input port of this filter.
     * 
     * @param event
     *            The next event.
     */
    @InputPort(name = INPUT_PORT_NAME_INVALID_TRACES, eventTypes = { Object.class }, description = "Receives incoming objects to be forwarded to a queue")
    public final void inputInvalidTraces(final Object monitoringRecord) {
        try {
            // if (monitoringRecord instanceof IMonitoringRecord) {
            // byte[] message2 = toByteArray((IMonitoringRecord)
            // monitoringRecord);

            final ByteArrayOutputStream boas = new ByteArrayOutputStream();

            final ObjectOutputStream out = new ObjectOutputStream(boas);
            out.writeObject(monitoringRecord);
            out.close();

            final byte[] message2 = boas.toByteArray(); // TODO optimize

            System.arraycopy(message2, 0, invalidTracesMessages,
                    invalidTracesMessagesOffset, message2.length);
            invalidTracesMessagesOffset += message2.length;

            if ((invalidTracesMessagesOffset > (MESSAGE_BUFFER_SIZE - MESSAGE_BUFFER_SIZE))) { // TODO
                // unsafe
                // 200
                // ||
                // writeQueue.isEmpty()
                UnsafeBits.putInt(invalidTracesMessages,
                        invalidTracesMessagesOffset, -1);
                sendMessage(invalidTracesMessages, "invalidTracesMaster");
                invalidTracesMessagesOffset = 0;
            }
        }
        catch (final IOException e) {
            LOG.error("Error sending record", e);
        }
    }

    @SuppressWarnings("unused")
    private byte[] toByteArray(final IMonitoringRecord monitoringRecord) {
        return new byte[1]; // TODO
    }

    private void sendMessage(final byte[] message, final String queueName)
            throws IOException {

        synchronized (this) {}
    }

    protected final void cleanup() {
        disconnect();
    }

    private void disconnect() {

    }

    @Override
    public final String toString() {
        final StringBuilder sb = new StringBuilder(128);

        return sb.toString();
    }

    @Override
    public Configuration getCurrentConfiguration() {
        final Configuration configuration = new Configuration();
        configuration.setProperty(CONFIG_PROPERTY_NAME_PROVIDER, providerUrl);
        return configuration;
    }
}
